@@ -1,4 +1,4 @@
-#![cfg_attr(RUSTC_WITH_SPECIALIZATION, feature(specialization))]
+#![cfg_attr(RUSTC_WITH_SPECIALIZATION, feature(min_specialization))]
 #![allow(clippy::integer_arithmetic)]
 
 pub mod authorized_voters;
@@ -12,4 +12,4 @@ extern crate solana_metrics;
 #[macro_use]
 extern crate safecoin_frozen_abi_macro;
 
-pub use solana_sdk::vote::program::{check_id, id};
+pub use safecoin_sdk::vote::program::{check_id, id};
